/*
 * @(#)WinTray.java	1.00 06/09/03
 *
 * Copyright 2003 AVIA Inc. All rights reserved.
 * AVIA PROPRIETARY/CONFIDENTIAL. Use is subject to license terms.
 */

package alexiworld.yahoo.com;

/**
 * The WinTray class is designed to provide Win32 Tray functionality 
 * for java applications.
 * <p>
 * In addition, this class provides many methods for 
 * adding, modifying and deleting of menu items in Win32 Tray, as well as 
 * events when some of them is pressed.
 * To registrate as listener for these events implement WinTrayListener interface
 * and use WinTray's <code>addWinTrayListener</code> method.
 *
 * @author Alexi Jordanov (alexiworld@yahoo.com)
 * @version 1.00 06/09/03
 * @since J2Exe 2.0
 */
 
import java.util.Vector;
import java.util.Enumeration;

public class WinTray {
	
	static Vector lst;

	static {
		try {
			String exePath = System.getProperty("application.exe");
			//System.out.println(exePath);
			System.load(exePath);
			lst = new Vector();
		} catch (Throwable t) {
			lst = null;
		}
	}
	
	public static boolean isWinTrayAvailable() {
		return (lst != null);
	}
	
	public static void addWinTrayListener(WinTrayListener wtl) {
		if (!isWinTrayAvailable()) return;
		synchronized (lst) {
			if (lst.indexOf(wtl) < 0) lst.addElement(wtl);
		}
	}
	
	public static void removeWinTrayListener(WinTrayListener wtl) {
		if (!isWinTrayAvailable()) return;
		synchronized (lst) {
			int idx = lst.indexOf(wtl);
			if (idx >= 0) lst.removeElementAt(idx);
		}
	}

	/**
	 * Automatically minimize application into tray bar.
	 */
	public static native void minimize();
	
	/**
	 * Automatically minimize application into tray bar.
	 */
	public static native void restore();
	
	/**
	 * @return count of created menu icons.
	 */
	public static native int getTrayMenuIconsCount();
	
	/**
	 * Add new menu icon. It can be later assigned to any menu item.
	 * 
	 * @param icoID			icon id
	 * @param nWidth		icon width
	 * @param nHeight		icon height		
	 * @param coldata		array containing color data. each 3-byte triplet 
	 * in the array represents the relative intensities of red, blue, green, 
	 * respectively, for a pixel.
	 */
	public static native void addTrayMenuIcon(int icoID, int nWidth, int nHeight, byte coldata[]);
	
	/**
	 * Remove menu icon.
	 * 
	 * @param icoID		icon id
	 */
	public static native boolean removeTrayMenuIcon(int icoID);
	
	/**
	 *	Removes all menu icons. 
	 */
	public static native void removeAllTrayMenuIcons();
	
	/**
	 * @return count of created menu items.
	 */
	public static native int getTrayMenuItemsCount();
	
	/**
	 * Add new menu item into tray popup menu.
	 * @param name		menu item name
	 * @param cmdID		command id
	 * @param icoID		icon id (use -1 to dismiss icon)
	 */
	public static native void addTrayMenuItem(String name, int cmdID, int icoID);
	
	/**
	 * Insert new menu item into tray popup menu.
	 * 
	 * @param name		menu item name
	 * @param cmdID		command id
	 * @param icoID		icon id (use -1 to dismiss icon)
	 * @param pos		position, where to be inserted
	 * @return	true if operation was successful, false otherwise
	 */
	public static native boolean insertTrayMenuItem(String name, int cmdID, int icoID, int pos);
	
	/**
	 * Remove menu item by its name in tray popup menu.
	 * 
	 * @param name		menu item name
	 * @return	true if operation was successful, false otherwise
	 */
	public static native boolean removeTrayMenuItem(String name);
	
	/**
	 * Remove menu item by its command id in tray popup menu.
	 * 
	 * @param cmdID		command id
	 * @return	true if operation was successful, false otherwise
	 */
	public static native boolean removeTrayMenuItemByID(int cmdID);
	
	/**
	 * Remove menu item by its position in tray popup menu.
	 * 
	 * @param pos		position of menu item
	 * @return	true if operation was successful, false otherwise
	 */
	public static native boolean removeTrayMenuItemByPos(int pos);
	
	/**
	 * Removes all menu items from tray popup menu.
	 */
	public static native void removeAllTrayMenuItems();
	
	/**
	 * Enable/Disable menu item by its name in tray popup menu.
	 * 
	 * @param name		menu item name
	 * @param state		true to enable, false to make it gray
	 */
	public static native void enableTrayMenuItem(String name, boolean state);
	
	/**
	 * Enable/Disable menu item by its command id in tray popup menu.
	 * 
	 * @param cmdID		command id
	 * @param state		true to enable, false to make it gray
	 */
	public static native void  enableTrayMenuItemByID(int cmdID, boolean state);
	
	/**
	 * Enable/Disable menu item by its position in tray popup menu.
	 * 
	 * @param pos		position of menu item
	 * @param state		true to enable, false to make it gray
	 */
	public static native void  enableTrayMenuItemByPos(int pos, boolean state);

	/**
	 * Check whether specified menu item is enabled.
	 * 
	 * @param name		menu item name
	 * @return	true if menu item is enabled, false otherwise
	 */
	public static native boolean isTrayMenuItemEnabled(String name);
	
	/**
	 * Check whether specified menu item is enabled.
	 * 
	 * @param cmdID		command id
	 * @return	true if menu item is enabled, false otherwise
	 */
	public static native boolean isTrayMenuItemEnabledByID(int cmdID);
	
	/**
	 * Check whether specified menu item is enabled.
	 * 
	 * @param pos		position of menu item
	 * @return	true if menu item is enabled, false otherwise
	 */
	public static native boolean isTrayMenuItemEnabledByPos(int pos);

	/**
	 * Check/Uncheck menu item by its name from tray popup menu.
	 * Note that check/uncheck operations takes precedence in icon 
	 * of menu item. So do not consider as a bug when you don't see
	 * menu icon.
	 * 
	 * @param name		menu item name
	 * @param state		true to check it, false to make it unchecked
	 */
	public static native void checkTrayMenuItem(String name, boolean state);
	
	/**
	 * Remove menu item by its command id from tray popup menu.
	 * Note that check/uncheck operations takes precedence in icon 
	 * of menu item. So do not consider as a bug when you don't see
	 * menu icon.
	 * 
	 * @param cmdID		command id
	 * @param state		true to check it, false to make it unchecked
	 */
	public static native void checkTrayMenuItemByID(int cmdID, boolean state);
	
	/**
	 * Remove menu item by its position from tray popup menu.
	 * Note that check/uncheck operations takes precedence in icon 
	 * of menu item. So do not consider as a bug when you don't see
	 * menu icon.
	 * 
	 * @param pos		position of menu item
	 * @param state		true to check it, false to make it unchecked
	 */
	public static native void checkTrayMenuItemByPos(int pos, boolean state);
	
	/**
	 * Check whether specified menu item is enabled.
	 * 
	 * @param name		menu item name
	 * @return	true if menu item is checked, false otherwise
	 */
	public static native boolean isTrayMenuItemChecked(String name);
	
	/**
	 * Check whether specified menu item is enabled.
	 * 
	 * @param cmdID		command id
	 * @return	true if menu item is checked, false otherwise
	 */
	public static native boolean isTrayMenuItemCheckedByID(int cmdID);
	
	/**
	 * Check whether specified menu item is enabled.
	 * 
	 * @param pos		position of menu item
	 * @return	true if menu item is checked, false otherwise
	 */
	public static native boolean isTrayMenuItemCheckedByPos(int pos);

	public static void actionPerformed(int code) {
		//System.out.println("[event]: " + code);
		Enumeration e = lst.elements();
		while (e.hasMoreElements()) {
			WinTrayListener wtl = (WinTrayListener) e.nextElement();
			wtl.actionPerformed(code);
		}
	}
	
	public static void main(String[] args) {
		final javax.swing.JFrame frame = new javax.swing.JFrame();

		if (isWinTrayAvailable()) {
			WinTrayListener wtl = new WinTrayListener()  {
				public void actionPerformed(int code) {
					javax.swing.JFrame framex = new javax.swing.JFrame();
					framex.setTitle("koko");
					framex.setSize(new java.awt.Dimension(300, 200));
					framex.setVisible(true);
					framex = null;
					//javax.swing.JOptionPane.showMessageDialog(frame, "koko");
					System.out.println("event: " + code);
					if (code == 0) System.exit(0);
				} 
			};
			addWinTrayListener(wtl);
			addTrayMenuItem("java action 1", 100, -1);
			addTrayMenuItem(null, 101, -1);
			addTrayMenuItem("java action 2", 102, -1);
			addTrayMenuItem(null, 103, -1);
			addTrayMenuItem("java action 3", 104, -1);
		}
		
		frame.setSize(new java.awt.Dimension(300, 200));
		frame.setVisible(true);
		
		// ATTENTION: Setting of title is necessary for GUI applications only. Console applications does not need such as.
		// ATTENTION: Setting of title is recommended for GUI applications in order to specify the frame which closing 
		// should minimize the applicaion in tray. It might be omited for AWT/Swing of JDK1.4.x because J2Exe automatically 
		// catch first window. But for AWT/Swing of JDK1.2.2 (and may be some others GUI libraries + JDK versions) it is 
		// obligatory because it creates fake invisible window called "bufferCreator" before creating frame.

		frame.setTitle("WinTray"); // marks frame which closing should minimize applicaion in tray 
		frame.setTitle("Sample Application"); // restore original frame title

		frame.addWindowListener(
			new java.awt.event.WindowListener() {
				public void windowActivated(java.awt.event.WindowEvent e) {}
				public void windowClosed(java.awt.event.WindowEvent e) {}
				public void windowClosing(java.awt.event.WindowEvent e) { System.exit(0); } // senceless for Win32
				public void windowDeactivated(java.awt.event.WindowEvent e) {}
				public void windowDeiconified(java.awt.event.WindowEvent e) {}
				public void windowIconified(java.awt.event.WindowEvent e) {}
				public void windowOpened(java.awt.event.WindowEvent e) {}			
			}
		);		
	}
	
}
